@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2018 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2023 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -91,6 +91,17 @@ private:
     ////////////////////////////////////////////////////////////
     virtual bool activate(bool active);
 
+    ////////////////////////////////////////////////////////////
+    /// \brief Tell if the render-texture will use sRGB encoding when drawing on it
+    ///
+    /// You can request sRGB encoding for a render-texture
+    /// by having the sRgbCapable flag set for the context parameter of create() method
+    ///
+    /// \return True if the render-texture use sRGB encoding, false otherwise
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual bool isSrgb() const;
+
     ////////////////////////////////////////////////////////////
     /// \brief Update the pixels of the target texture
     ///
@@ -102,9 +113,9 @@ private:
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    Context*     m_context; ///< P-Buffer based context
-    unsigned int m_width;   ///< Width of the P-Buffer
-    unsigned int m_height;  ///< Height of the P-Buffer
+    Context*     m_context; //!< P-Buffer based context
+    unsigned int m_width;   //!< Width of the P-Buffer
+    unsigned int m_height;  //!< Height of the P-Buffer
 };
 
 } // namespace priv
