@@ -49,7 +49,7 @@ namespace priv
 WindowImplAndroid* WindowImplAndroid::singleInstance = NULL;
 
 ////////////////////////////////////////////////////////////
-WindowImplAndroid::WindowImplAndroid(WindowHandle handle)
+WindowImplAndroid::WindowImplAndroid(WindowHandle /* handle */)
 : m_size(0, 0)
 , m_windowBeingCreated(false)
 , m_windowBeingDestroyed(false)
@@ -59,41 +59,42 @@ WindowImplAndroid::WindowImplAndroid(WindowHandle handle)
 
 
 ////////////////////////////////////////////////////////////
-WindowImplAndroid::WindowImplAndroid(VideoMode mode, const String& title, unsigned long style, const ContextSettings& settings)
+WindowImplAndroid::WindowImplAndroid(VideoMode mode, const String& /* title */, unsigned long style, const ContextSettings& /* settings */)
 : m_size(mode.width, mode.height)
 , m_windowBeingCreated(false)
 , m_windowBeingDestroyed(false)
 , m_hasFocus(false)
 {
-    ActivityStates* states = getActivity(NULL);
-    Lock lock(states->mutex);
+    priv::ActivityStates& states = priv::getActivity();
+    Lock lock(states.mutex);
 
     if (style& Style::Fullscreen)
-        states->fullscreen = true;
+        states.fullscreen = true;
 
     WindowImplAndroid::singleInstance = this;
-    states->forwardEvent = forwardEvent;
+    states.forwardEvent = forwardEvent;
 
     // Register process event callback
-    states->processEvent = processEvent;
+    states.processEvent = processEvent;
 
-    states->initialized = true;
+    states.initialized = true;
 }
 
 
 ////////////////////////////////////////////////////////////
 WindowImplAndroid::~WindowImplAndroid()
 {
+    WindowImplAndroid::singleInstance = NULL;
 }
 
 
 ////////////////////////////////////////////////////////////
 WindowHandle WindowImplAndroid::getSystemHandle() const
 {
-    ActivityStates* states = getActivity(NULL);
-    Lock lock(states->mutex);
+    ActivityStates& states = getActivity();
+    Lock lock(states.mutex);
 
-    return states->window;
+    return states.window;
 }
 
 
@@ -103,22 +104,22 @@ void WindowImplAndroid::processEvents()
     // Process incoming OS events
     ALooper_pollAll(0, NULL, NULL, NULL);
 
-    ActivityStates* states = getActivity(NULL);
-    sf::Lock lock(states->mutex);
+    ActivityStates& states = getActivity();
+    Lock lock(states.mutex);
 
     if (m_windowBeingCreated)
     {
-        states->context->createSurface(states->window);
+        states.context->createSurface(states.window);
         m_windowBeingCreated = false;
     }
 
     if (m_windowBeingDestroyed)
     {
-        states->context->destroySurface();
+        states.context->destroySurface();
         m_windowBeingDestroyed = false;
     }
 
-    states->updated = true;
+    states.updated = true;
 }
 
 
@@ -131,7 +132,7 @@ Vector2i WindowImplAndroid::getPosition() const
 
 
 ////////////////////////////////////////////////////////////
-void WindowImplAndroid::setPosition(const Vector2i& position)
+void WindowImplAndroid::setPosition(const Vector2i& /* position */)
 {
     // Not applicable
 }
@@ -145,55 +146,55 @@ Vector2u WindowImplAndroid::getSize() const
 
 
 ////////////////////////////////////////////////////////////
-void WindowImplAndroid::setSize(const Vector2u& size)
+void WindowImplAndroid::setSize(const Vector2u& /* size */)
 {
 }
 
 
 ////////////////////////////////////////////////////////////
-void WindowImplAndroid::setTitle(const String& title)
+void WindowImplAndroid::setTitle(const String& /* title */)
 {
     // Not applicable
 }
 
 
 ////////////////////////////////////////////////////////////
-void WindowImplAndroid::setIcon(unsigned int width, unsigned int height, const Uint8* pixels)
+void WindowImplAndroid::setIcon(unsigned int /* width */, unsigned int /* height */, const Uint8* /* pixels */)
 {
     // Not applicable
 }
 
 
 ////////////////////////////////////////////////////////////
-void WindowImplAndroid::setVisible(bool visible)
+void WindowImplAndroid::setVisible(bool /* visible */)
 {
     // Not applicable
 }
 
 
 ////////////////////////////////////////////////////////////
-void WindowImplAndroid::setMouseCursorVisible(bool visible)
+void WindowImplAndroid::setMouseCursorVisible(bool /* visible */)
 {
     // Not applicable
 }
 
 
 ////////////////////////////////////////////////////////////
-void WindowImplAndroid::setMouseCursorGrabbed(bool grabbed)
+void WindowImplAndroid::setMouseCursorGrabbed(bool /* grabbed */)
 {
     // Not applicable
 }
 
 
 ////////////////////////////////////////////////////////////
-void WindowImplAndroid::setMouseCursor(const CursorImpl& cursor)
+void WindowImplAndroid::setMouseCursor(const CursorImpl& /* cursor */)
 {
     // Not applicable
 }
 
 
 ////////////////////////////////////////////////////////////
-void WindowImplAndroid::setKeyRepeatEnabled(bool enabled)
+void WindowImplAndroid::setKeyRepeatEnabled(bool /* enabled */)
 {
     // Not applicable
 }
@@ -216,36 +217,39 @@ bool WindowImplAndroid::hasFocus() const
 ////////////////////////////////////////////////////////////
 void WindowImplAndroid::forwardEvent(const Event& event)
 {
-    ActivityStates* states = getActivity(NULL);
-
-    if (event.type == Event::GainedFocus)
+    if (WindowImplAndroid::singleInstance != NULL)
     {
-        WindowImplAndroid::singleInstance->m_size.x = ANativeWindow_getWidth(states->window);
-        WindowImplAndroid::singleInstance->m_size.y = ANativeWindow_getHeight(states->window);
-        WindowImplAndroid::singleInstance->m_windowBeingCreated = true;
-        WindowImplAndroid::singleInstance->m_hasFocus = true;
-    }
-    else if (event.type == Event::LostFocus)
-    {
-        WindowImplAndroid::singleInstance->m_windowBeingDestroyed = true;
-        WindowImplAndroid::singleInstance->m_hasFocus = false;
-    }
+        ActivityStates& states = getActivity();
 
-    WindowImplAndroid::singleInstance->pushEvent(event);
+        if (event.type == Event::GainedFocus)
+        {
+            WindowImplAndroid::singleInstance->m_size.x = static_cast<unsigned int>(ANativeWindow_getWidth(states.window));
+            WindowImplAndroid::singleInstance->m_size.y = static_cast<unsigned int>(ANativeWindow_getHeight(states.window));
+            WindowImplAndroid::singleInstance->m_windowBeingCreated = true;
+            WindowImplAndroid::singleInstance->m_hasFocus = true;
+        }
+        else if (event.type == Event::LostFocus)
+        {
+            WindowImplAndroid::singleInstance->m_windowBeingDestroyed = true;
+            WindowImplAndroid::singleInstance->m_hasFocus = false;
+        }
+
+        WindowImplAndroid::singleInstance->pushEvent(event);
+    }
 }
 
 
 ////////////////////////////////////////////////////////////
-int WindowImplAndroid::processEvent(int fd, int events, void* data)
+int WindowImplAndroid::processEvent(int /* fd */, int /* events */, void* /* data */)
 {
-    ActivityStates* states = getActivity(NULL);
-    Lock lock(states->mutex);
+    ActivityStates& states = getActivity();
+    Lock lock(states.mutex);
 
     AInputEvent* _event = NULL;
 
-    if (AInputQueue_getEvent(states->inputQueue, &_event) >= 0)
+    if (AInputQueue_getEvent(states.inputQueue, &_event) >= 0)
     {
-        if (AInputQueue_preDispatchEvent(states->inputQueue, _event))
+        if (AInputQueue_preDispatchEvent(states.inputQueue, _event))
             return 1;
 
         int handled = 0;
@@ -302,7 +306,7 @@ int WindowImplAndroid::processEvent(int fd, int events, void* data)
 
         }
 
-        AInputQueue_finishEvent(states->inputQueue, _event, handled);
+        AInputQueue_finishEvent(states.inputQueue, _event, handled);
     }
 
     return 1;
@@ -310,14 +314,13 @@ int WindowImplAndroid::processEvent(int fd, int events, void* data)
 
 
 ////////////////////////////////////////////////////////////
-int WindowImplAndroid::processScrollEvent(AInputEvent* _event, ActivityStates* states)
+int WindowImplAndroid::processScrollEvent(AInputEvent* _event, ActivityStates& states)
 {
     // Prepare the Java virtual machine
     jint lResult;
-    jint lFlags = 0;
 
-    JavaVM* lJavaVM = states->activity->vm;
-    JNIEnv* lJNIEnv = states->activity->env;
+    JavaVM* lJavaVM = states.activity->vm;
+    JNIEnv* lJNIEnv = states.activity->env;
 
     JavaVMAttachArgs lJavaVMAttachArgs;
     lJavaVMAttachArgs.version = JNI_VERSION_1_6;
@@ -332,23 +335,33 @@ int WindowImplAndroid::processScrollEvent(AInputEvent* _event, ActivityStates* s
     }
 
     // Retrieve everything we need to create this MotionEvent in Java
-    jlong downTime = AMotionEvent_getDownTime(_event);
-    jlong eventTime = AMotionEvent_getEventTime(_event);
-    jint action = AMotionEvent_getAction(_event);
-    jfloat x = AMotionEvent_getX(_event, 0);
-    jfloat y = AMotionEvent_getY(_event, 0);
-    jfloat pressure = AMotionEvent_getPressure(_event, 0);
-    jfloat size = AMotionEvent_getSize(_event, 0);
-    jint metaState = AMotionEvent_getMetaState(_event);
-    jfloat xPrecision = AMotionEvent_getXPrecision(_event);
-    jfloat yPrecision = AMotionEvent_getYPrecision(_event);
-    jint deviceId = AInputEvent_getDeviceId(_event);
-    jint edgeFlags = AMotionEvent_getEdgeFlags(_event);
+    Int64 downTime = AMotionEvent_getDownTime(_event);
+    Int64 eventTime = AMotionEvent_getEventTime(_event);
+    Int32 action = AMotionEvent_getAction(_event);
+    float x = AMotionEvent_getX(_event, 0);
+    float y = AMotionEvent_getY(_event, 0);
+    float pressure = AMotionEvent_getPressure(_event, 0);
+    float size = AMotionEvent_getSize(_event, 0);
+    Int32 metaState = AMotionEvent_getMetaState(_event);
+    float xPrecision = AMotionEvent_getXPrecision(_event);
+    float yPrecision = AMotionEvent_getYPrecision(_event);
+    Int32 deviceId = AInputEvent_getDeviceId(_event);
+    Int32 edgeFlags = AMotionEvent_getEdgeFlags(_event);
 
     // Create the MotionEvent object in Java trough its static constructor obtain()
     jclass ClassMotionEvent = lJNIEnv->FindClass("android/view/MotionEvent");
     jmethodID StaticMethodObtain = lJNIEnv->GetStaticMethodID(ClassMotionEvent, "obtain", "(JJIFFFFIFFII)Landroid/view/MotionEvent;");
-    jobject ObjectMotionEvent = lJNIEnv->CallStaticObjectMethod(ClassMotionEvent, StaticMethodObtain, downTime, eventTime, action, x, y, pressure, size, metaState, xPrecision, yPrecision, deviceId, edgeFlags);
+    // Note: C standard compatibility, varargs
+    //       automatically promote floats to doubles
+    //       even though the function signature declares float
+    jobject ObjectMotionEvent = lJNIEnv->CallStaticObjectMethod(
+        ClassMotionEvent, StaticMethodObtain, downTime,
+        eventTime, action, static_cast<double>(x),
+        static_cast<double>(y), static_cast<double>(pressure),
+        static_cast<double>(size), metaState,
+        static_cast<double>(xPrecision), static_cast<double>(yPrecision),
+        deviceId, edgeFlags
+    );
 
     // Call its getAxisValue() method to get the delta value of our wheel move event
     jmethodID MethodGetAxisValue = lJNIEnv->GetMethodID(ClassMotionEvent, "getAxisValue", "(I)F");
@@ -360,9 +373,9 @@ int WindowImplAndroid::processScrollEvent(AInputEvent* _event, ActivityStates* s
     // Create and send our mouse wheel event
     Event event;
     event.type = Event::MouseWheelMoved;
-    event.mouseWheel.delta = static_cast<double>(delta);
-    event.mouseWheel.x = AMotionEvent_getX(_event, 0);
-    event.mouseWheel.y = AMotionEvent_getY(_event, 0);
+    event.mouseWheel.delta = static_cast<int>(delta);
+    event.mouseWheel.x = static_cast<int>(AMotionEvent_getX(_event, 0));
+    event.mouseWheel.y = static_cast<int>(AMotionEvent_getY(_event, 0));
 
     forwardEvent(event);
 
@@ -374,9 +387,8 @@ int WindowImplAndroid::processScrollEvent(AInputEvent* _event, ActivityStates* s
 
 
 ////////////////////////////////////////////////////////////
-int WindowImplAndroid::processKeyEvent(AInputEvent* _event, ActivityStates* states)
+int WindowImplAndroid::processKeyEvent(AInputEvent* _event, ActivityStates& /* states */)
 {
-    int32_t device = AInputEvent_getSource(_event);
     int32_t action = AKeyEvent_getAction(_event);
 
     int32_t key = AKeyEvent_getKeyCode(_event);
@@ -398,10 +410,10 @@ int WindowImplAndroid::processKeyEvent(AInputEvent* _event, ActivityStates* stat
         event.type = Event::KeyReleased;
         forwardEvent(event);
 
-        if (int unicode = getUnicode(_event))
+        if (Uint32 unicode = static_cast<Uint32>(getUnicode(_event)))
         {
             event.type = Event::TextEntered;
-            event.text.unicode = unicode;
+            event.text.unicode = static_cast<Uint32>(unicode);
             forwardEvent(event);
         }
         return 1;
@@ -421,10 +433,10 @@ int WindowImplAndroid::processKeyEvent(AInputEvent* _event, ActivityStates* stat
             // https://code.google.com/p/android/issues/detail?id=33998
             return 0;
         }
-        else if (int unicode = getUnicode(_event)) // This is a repeated sequence
+        else if (Uint32 unicode = static_cast<Uint32>(getUnicode(_event))) // This is a repeated sequence
         {
             event.type = Event::TextEntered;
-            event.text.unicode = unicode;
+            event.text.unicode = static_cast<Uint32>(unicode);
 
             int32_t repeats = AKeyEvent_getRepeatCount(_event);
             for (int32_t i = 0; i < repeats; ++i)
@@ -438,44 +450,43 @@ int WindowImplAndroid::processKeyEvent(AInputEvent* _event, ActivityStates* stat
 
 
 ////////////////////////////////////////////////////////////
-int WindowImplAndroid::processMotionEvent(AInputEvent* _event, ActivityStates* states)
+int WindowImplAndroid::processMotionEvent(AInputEvent* _event, ActivityStates& states)
 {
     int32_t device = AInputEvent_getSource(_event);
-    int32_t action = AMotionEvent_getAction(_event);
 
     Event event;
 
     if (device == AINPUT_SOURCE_MOUSE)
         event.type = Event::MouseMoved;
-    else if (device & AINPUT_SOURCE_TOUCHSCREEN)
+    else if (static_cast<Uint32>(device) & AINPUT_SOURCE_TOUCHSCREEN)
         event.type = Event::TouchMoved;
 
-    int pointerCount = AMotionEvent_getPointerCount(_event);
+    size_t pointerCount = AMotionEvent_getPointerCount(_event);
 
-    for (int p = 0; p < pointerCount; p++)
+    for (size_t p = 0; p < pointerCount; p++)
     {
-        int id = AMotionEvent_getPointerId(_event, p);
+        int32_t id = AMotionEvent_getPointerId(_event, p);
 
-        float x = AMotionEvent_getX(_event, p);
-        float y = AMotionEvent_getY(_event, p);
+        int x = static_cast<int>(AMotionEvent_getX(_event, p));
+        int y = static_cast<int>(AMotionEvent_getY(_event, p));
 
         if (device == AINPUT_SOURCE_MOUSE)
         {
             event.mouseMove.x = x;
             event.mouseMove.y = y;
 
-            states->mousePosition = Vector2i(event.mouseMove.x, event.mouseMove.y);
+            states.mousePosition = Vector2i(event.mouseMove.x, event.mouseMove.y);
         }
-        else if (device & AINPUT_SOURCE_TOUCHSCREEN)
+        else if (static_cast<Uint32>(device) & AINPUT_SOURCE_TOUCHSCREEN)
         {
-            if (states->touchEvents[id].x == x && states->touchEvents[id].y == y)
+            if (states.touchEvents[id].x == x && states.touchEvents[id].y == y)
                 continue;
 
-            event.touch.finger = id;
+            event.touch.finger = static_cast<unsigned int>(id);
             event.touch.x = x;
             event.touch.y = y;
 
-            states->touchEvents[id] = Vector2i(event.touch.x, event.touch.y);
+            states.touchEvents[id] = Vector2i(event.touch.x, event.touch.y);
         }
 
         forwardEvent(event);
@@ -485,16 +496,16 @@ int WindowImplAndroid::processMotionEvent(AInputEvent* _event, ActivityStates* s
 
 
 ////////////////////////////////////////////////////////////
-int WindowImplAndroid::processPointerEvent(bool isDown, AInputEvent* _event, ActivityStates* states)
+int WindowImplAndroid::processPointerEvent(bool isDown, AInputEvent* _event, ActivityStates& states)
 {
     int32_t device = AInputEvent_getSource(_event);
     int32_t action = AMotionEvent_getAction(_event);
 
-    int index = (action & AMOTION_EVENT_ACTION_POINTER_INDEX_MASK) >> AMOTION_EVENT_ACTION_POINTER_INDEX_SHIFT;
-    int id = AMotionEvent_getPointerId(_event, index);
+    size_t index = (action & AMOTION_EVENT_ACTION_POINTER_INDEX_MASK) >> AMOTION_EVENT_ACTION_POINTER_INDEX_SHIFT;
+    int32_t id = AMotionEvent_getPointerId(_event, index);
 
-    float x = AMotionEvent_getX(_event, index);
-    float y = AMotionEvent_getY(_event, index);
+    int x = static_cast<int>(AMotionEvent_getX(_event, index));
+    int y = static_cast<int>(AMotionEvent_getY(_event, index));
 
     Event event;
 
@@ -508,16 +519,16 @@ int WindowImplAndroid::processPointerEvent(bool isDown, AInputEvent* _event, Act
             event.mouseButton.y = y;
 
             if (id >= 0 && id < Mouse::ButtonCount)
-                states->isButtonPressed[id] = true;
+                states.isButtonPressed[id] = true;
         }
-        else if (device & AINPUT_SOURCE_TOUCHSCREEN)
+        else if (static_cast<unsigned int>(device) & AINPUT_SOURCE_TOUCHSCREEN)
         {
             event.type = Event::TouchBegan;
-            event.touch.finger = id;
+            event.touch.finger = static_cast<unsigned int>(id);
             event.touch.x = x;
             event.touch.y = y;
 
-            states->touchEvents[id] = Vector2i(event.touch.x, event.touch.y);
+            states.touchEvents[id] = Vector2i(event.touch.x, event.touch.y);
         }
     }
     else
@@ -530,16 +541,16 @@ int WindowImplAndroid::processPointerEvent(bool isDown, AInputEvent* _event, Act
             event.mouseButton.y = y;
 
             if (id >= 0 && id < Mouse::ButtonCount)
-                states->isButtonPressed[id] = false;
+                states.isButtonPressed[id] = false;
         }
-        else if (device & AINPUT_SOURCE_TOUCHSCREEN)
+        else if (static_cast<Uint32>(device) & AINPUT_SOURCE_TOUCHSCREEN)
         {
             event.type = Event::TouchEnded;
-            event.touch.finger = id;
+            event.touch.finger = static_cast<unsigned int>(id);
             event.touch.x = x;
             event.touch.y = y;
 
-            states->touchEvents.erase(id);
+            states.touchEvents.erase(id);
         }
     }
 
@@ -622,7 +633,7 @@ Keyboard::Key WindowImplAndroid::androidKeyToSF(int32_t key)
         case AKEYCODE_ENTER:              return Keyboard::Enter;
         case AKEYCODE_DEL:                return Keyboard::Backspace;
         case AKEYCODE_FORWARD_DEL:        return Keyboard::Delete;
-        case AKEYCODE_GRAVE:              return Keyboard::Tilde;
+        case AKEYCODE_GRAVE:              return Keyboard::Grave;
         case AKEYCODE_MINUS:              return Keyboard::Subtract;
         case AKEYCODE_EQUALS:             return Keyboard::Equal;
         case AKEYCODE_LEFT_BRACKET:       return Keyboard::LBracket;
@@ -664,7 +675,8 @@ Keyboard::Key WindowImplAndroid::androidKeyToSF(int32_t key)
         case AKEYCODE_BUTTON_THUMBR:
         case AKEYCODE_BUTTON_START:
         case AKEYCODE_BUTTON_SELECT:
-        case AKEYCODE_BUTTON_MODE:        return Keyboard::Unknown;
+        case AKEYCODE_BUTTON_MODE:
+        default:                          return Keyboard::Unknown;
     }
 }
 
@@ -673,15 +685,14 @@ Keyboard::Key WindowImplAndroid::androidKeyToSF(int32_t key)
 int WindowImplAndroid::getUnicode(AInputEvent* event)
 {
     // Retrieve activity states
-    ActivityStates* states = getActivity(NULL);
-    Lock lock(states->mutex);
+    ActivityStates& states = getActivity();
+    Lock lock(states.mutex);
 
     // Initializes JNI
     jint lResult;
-    jint lFlags = 0;
 
-    JavaVM* lJavaVM = states->activity->vm;
-    JNIEnv* lJNIEnv = states->activity->env;
+    JavaVM* lJavaVM = states.activity->vm;
+    JNIEnv* lJNIEnv = states.activity->env;
 
     JavaVMAttachArgs lJavaVMAttachArgs;
     lJavaVMAttachArgs.version = JNI_VERSION_1_6;
