@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2018 Marco Antognini (antognini.marco@gmail.com),
+// Copyright (C) 2007-2023 Marco Antognini (antognini.marco@gmail.com),
 //                         Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
@@ -26,10 +26,10 @@
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
+#include <SFML/Window/OSX/AutoreleasePoolWrapper.hpp>
 #include <SFML/Window/OSX/WindowImplCocoa.hpp>
 #include <SFML/System/Err.hpp>
 
-#import <SFML/Window/OSX/AutoreleasePoolWrapper.h>
 #import <SFML/Window/OSX/cpp_objc_conversion.h>
 #import <SFML/Window/OSX/Scaling.h>
 #import <SFML/Window/OSX/SFApplication.h>
@@ -59,6 +59,7 @@ namespace
 ////////////////////////////////////////////////////////
 void hideMouseCursor()
 {
+    AutoreleasePool pool;
     if (!isCursorHidden)
     {
         [NSCursor hide];
@@ -70,6 +71,7 @@ void hideMouseCursor()
 ////////////////////////////////////////////////////////
 void showMouseCursor()
 {
+    AutoreleasePool pool;
     if (isCursorHidden)
     {
         [NSCursor unhide];
@@ -84,11 +86,9 @@ void showMouseCursor()
 WindowImplCocoa::WindowImplCocoa(WindowHandle handle) :
 m_showCursor(true)
 {
-    // Ask for a pool.
-    ensureThreadHasPool();
-
+    AutoreleasePool pool;
     // Treat the handle as it real type
-    id nsHandle = (id)handle;
+    id nsHandle = static_cast<id>(handle);
     if ([nsHandle isKindOfClass:[NSWindow class]])
     {
         // We have a window.
@@ -126,15 +126,10 @@ WindowImplCocoa::WindowImplCocoa(VideoMode mode,
                                  const ContextSettings& /*settings*/) :
 m_showCursor(true)
 {
+    AutoreleasePool pool;
     // Transform the app process.
     setUpProcess();
 
-    // Ask for a pool.
-    ensureThreadHasPool();
-
-    // Use backing size
-    scaleInWidthHeight(mode, nil);
-
     m_delegate = [[SFWindowController alloc] initWithMode:mode andStyle:style];
     [m_delegate changeTitle:sfStringToNSString(title)];
     [m_delegate setRequesterTo:this];
@@ -147,24 +142,29 @@ m_showCursor(true)
 ////////////////////////////////////////////////////////////
 WindowImplCocoa::~WindowImplCocoa()
 {
+    AutoreleasePool pool;
     [m_delegate closeWindow];
-
+    // Tell the window/view controller (and the OpenGL view) that the delegate
+    // (this object) no longer exists to prevent events being sent to the window
+    // after it has been deleted.
+    [m_delegate setRequesterTo:0];
     [m_delegate release];
 
     // Put the next window in front, if any.
     NSArray* windows = [NSApp orderedWindows];
     if ([windows count] > 0)
-        [[windows objectAtIndex:0] makeKeyAndOrderFront:nil];
-
-    drainThreadPool(); // Make sure everything was freed
-    // This solve some issue when sf::Window::Create is called for the
-    // second time (nothing was render until the function was called again)
+    {
+        NSWindow *nextWindow = [windows objectAtIndex:0];
+        if ([nextWindow isVisible])
+            [nextWindow makeKeyAndOrderFront:nil];
+    }
 }
 
 
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::applyContext(NSOpenGLContextRef context) const
 {
+    AutoreleasePool pool;
     [m_delegate applyContext:context];
 }
 
@@ -172,6 +172,7 @@ void WindowImplCocoa::applyContext(NSOpenGLContextRef context) const
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::setUpProcess(void)
 {
+    AutoreleasePool pool;
     static bool isTheProcessSetAsApplication = false;
 
     if (!isTheProcessSetAsApplication)
@@ -303,7 +304,7 @@ void WindowImplCocoa::mouseWheelScrolledAt(float deltaX, float deltaY, int x, in
     Event event;
 
     event.type = Event::MouseWheelMoved;
-    event.mouseWheel.delta = deltaY;
+    event.mouseWheel.delta = static_cast<int>(deltaY);
     event.mouseWheel.x = x;
     event.mouseWheel.y = y;
     scaleOutXY(event.mouseWheel, m_delegate);
@@ -394,8 +395,8 @@ void WindowImplCocoa::textEntered(unichar charcode)
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::processEvents()
 {
+    AutoreleasePool pool;
     [m_delegate processEvent];
-    drainThreadPool(); // Reduce memory footprint
 }
 
 #pragma mark
@@ -404,6 +405,7 @@ void WindowImplCocoa::processEvents()
 ////////////////////////////////////////////////////////////
 WindowHandle WindowImplCocoa::getSystemHandle() const
 {
+    AutoreleasePool pool;
     return [m_delegate getSystemHandle];
 }
 
@@ -411,8 +413,9 @@ WindowHandle WindowImplCocoa::getSystemHandle() const
 ////////////////////////////////////////////////////////////
 Vector2i WindowImplCocoa::getPosition() const
 {
+    AutoreleasePool pool;
     NSPoint pos = [m_delegate position];
-    sf::Vector2i ret(pos.x, pos.y);
+    sf::Vector2i ret(static_cast<int>(pos.x), static_cast<int>(pos.y));
     scaleOutXY(ret, m_delegate);
     return ret;
 }
@@ -421,6 +424,7 @@ Vector2i WindowImplCocoa::getPosition() const
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::setPosition(const Vector2i& position)
 {
+    AutoreleasePool pool;
     sf::Vector2i backingPosition = position;
     scaleInXY(backingPosition, m_delegate);
     [m_delegate setWindowPositionToX:backingPosition.x Y:backingPosition.y];
@@ -430,8 +434,9 @@ void WindowImplCocoa::setPosition(const Vector2i& position)
 ////////////////////////////////////////////////////////////
 Vector2u WindowImplCocoa::getSize() const
 {
+    AutoreleasePool pool;
     NSSize size = [m_delegate size];
-    Vector2u ret(size.width, size.height);
+    Vector2u ret(static_cast<unsigned int>(size.width), static_cast<unsigned int>(size.height));
     scaleOutXY(ret, m_delegate);
     return ret;
 }
@@ -449,6 +454,7 @@ void WindowImplCocoa::setSize(const Vector2u& size)
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::setTitle(const String& title)
 {
+    AutoreleasePool pool;
     [m_delegate changeTitle:sfStringToNSString(title)];
 }
 
@@ -456,6 +462,7 @@ void WindowImplCocoa::setTitle(const String& title)
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::setIcon(unsigned int width, unsigned int height, const Uint8* pixels)
 {
+    AutoreleasePool pool;
     [m_delegate setIconTo:width by:height with:pixels];
 }
 
@@ -463,6 +470,7 @@ void WindowImplCocoa::setIcon(unsigned int width, unsigned int height, const Uin
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::setVisible(bool visible)
 {
+    AutoreleasePool pool;
     if (visible)
         [m_delegate showWindow];
     else
@@ -473,6 +481,7 @@ void WindowImplCocoa::setVisible(bool visible)
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::setMouseCursorVisible(bool visible)
 {
+    AutoreleasePool pool;
     m_showCursor = visible;
 
     // If the mouse is over the window, we apply the new setting
@@ -489,6 +498,7 @@ void WindowImplCocoa::setMouseCursorVisible(bool visible)
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::setMouseCursorGrabbed(bool grabbed)
 {
+    AutoreleasePool pool;
     [m_delegate setCursorGrabbed:grabbed];
 }
 
@@ -496,6 +506,7 @@ void WindowImplCocoa::setMouseCursorGrabbed(bool grabbed)
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::setMouseCursor(const CursorImpl& cursor)
 {
+    AutoreleasePool pool;
     [m_delegate setCursor:cursor.m_cursor];
 }
 
@@ -503,6 +514,7 @@ void WindowImplCocoa::setMouseCursor(const CursorImpl& cursor)
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::setKeyRepeatEnabled(bool enabled)
 {
+    AutoreleasePool pool;
     if (enabled)
         [m_delegate enableKeyRepeat];
     else
@@ -513,6 +525,7 @@ void WindowImplCocoa::setKeyRepeatEnabled(bool enabled)
 ////////////////////////////////////////////////////////////
 void WindowImplCocoa::requestFocus()
 {
+    AutoreleasePool pool;
     [m_delegate requestFocus];
 }
 
@@ -520,6 +533,7 @@ void WindowImplCocoa::requestFocus()
 ////////////////////////////////////////////////////////////
 bool WindowImplCocoa::hasFocus() const
 {
+    AutoreleasePool pool;
     return [m_delegate hasFocus];
 }
 
@@ -527,4 +541,3 @@ bool WindowImplCocoa::hasFocus() const
 } // namespace priv
 
 } // namespace sf
-
