@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2018 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2023 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -118,16 +118,16 @@ private:
     ////////////////////////////////////////////////////////////
     struct Item
     {
-        bool available;    ///< Is the sensor available on this device?
-        bool enabled;      ///< Current enable state of the sensor
-        SensorImpl sensor; ///< Sensor implementation
-        Vector3f value;    ///< The current sensor value
+        bool available;    //!< Is the sensor available on this device?
+        bool enabled;      //!< Current enable state of the sensor
+        SensorImpl sensor; //!< Sensor implementation
+        Vector3f value;    //!< The current sensor value
     };
 
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    Item m_sensors[Sensor::Count]; ///< Sensors information and state
+    Item m_sensors[Sensor::Count]; //!< Sensors information and state
 };
 
 } // namespace priv
