@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2018 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2023 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -29,6 +29,7 @@
 #include <SFML/System/Err.hpp>
 #include <linux/joystick.h>
 #include <libudev.h>
+#include <poll.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -155,12 +156,12 @@ namespace
                             // If not mapped before and it got added, map it now
                             const char* syspath = udev_device_get_syspath(udevDevice);
 
-                            JoystickRecord record;
-                            record.deviceNode = devnode;
-                            record.systemPath = syspath ? syspath : "";
-                            record.plugged = true;
+                            JoystickRecord newRecord;
+                            newRecord.deviceNode = devnode;
+                            newRecord.systemPath = syspath ? syspath : "";
+                            newRecord.plugged    = true;
 
-                            joystickList.push_back(record);
+                            joystickList.push_back(newRecord);
                         }
                         else if (std::strstr(action, "remove"))
                         {
@@ -211,12 +212,12 @@ namespace
 
         udev_list_entry_foreach(device, devices) {
             const char* syspath = udev_list_entry_get_name(device);
-            udev_device* udevDevice = udev_device_new_from_syspath(udevContext, syspath);
+            udev_device* newUdevDevice = udev_device_new_from_syspath(udevContext, syspath);
 
-            if (udevDevice && isJoystick(udevDevice))
+            if (newUdevDevice && isJoystick(newUdevDevice))
             {
                 // Since isJoystick returned true, this has to succeed
-                const char* devnode = udev_device_get_devnode(udevDevice);
+                const char* devnode = udev_device_get_devnode(newUdevDevice);
 
                 JoystickList::iterator record;
 
@@ -233,16 +234,16 @@ namespace
                 // If not mapped before, map it now
                 if (record == joystickList.end())
                 {
-                    JoystickRecord record;
-                    record.deviceNode = devnode;
-                    record.systemPath = syspath;
-                    record.plugged = true;
+                    JoystickRecord newRecord;
+                    newRecord.deviceNode = devnode;
+                    newRecord.systemPath = syspath;
+                    newRecord.plugged    = true;
 
-                    joystickList.push_back(record);
+                    joystickList.push_back(newRecord);
                 }
             }
 
-            udev_device_unref(udevDevice);
+            udev_device_unref(newUdevDevice);
         }
 
         udev_enumerate_unref(udevEnumerator);
@@ -253,13 +254,9 @@ namespace
         // This will not fail since we make sure udevMonitor is valid
         int monitorFd = udev_monitor_get_fd(udevMonitor);
 
-        fd_set descriptorSet;
-        FD_ZERO(&descriptorSet);
-        FD_SET(monitorFd, &descriptorSet);
-        timeval timeout = {0, 0};
+        pollfd fds{ monitorFd, POLLIN, 0 };
 
-        return (select(monitorFd + 1, &descriptorSet, NULL, NULL, &timeout) > 0) &&
-               FD_ISSET(monitorFd, &descriptorSet);
+        return (poll(&fds, 1, 0) > 0) && ((fds.revents & POLLIN) != 0);
     }
 
     // Get a property value from a udev device
@@ -602,7 +599,7 @@ JoystickCaps JoystickImpl::getCapabilities() const
     // Get the number of buttons
     char buttonCount;
     ioctl(m_file, JSIOCGBUTTONS, &buttonCount);
-    caps.buttonCount = buttonCount;
+    caps.buttonCount = static_cast<unsigned int>(buttonCount);
     if (caps.buttonCount > Joystick::ButtonCount)
         caps.buttonCount = Joystick::ButtonCount;
 
@@ -649,7 +646,7 @@ JoystickState JoystickImpl::JoystickImpl::update()
 
     // pop events from the joystick file
     js_event joyState;
-    int result = read(m_file, &joyState, sizeof(joyState));
+    ssize_t result = read(m_file, &joyState, sizeof(joyState));
     while (result > 0)
     {
         switch (joyState.type & ~JS_EVENT_INIT)
