@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2018 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2023 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -38,26 +38,49 @@
     #include <SFML/Window/Win32/WindowImplWin32.hpp>
     typedef sf::priv::WindowImplWin32 WindowImplType;
 
-#elif defined(SFML_SYSTEM_LINUX) || defined(SFML_SYSTEM_FREEBSD) || defined(SFML_SYSTEM_OPENBSD)
+    #include <SFML/Window/Win32/VulkanImplWin32.hpp>
+    typedef sf::priv::VulkanImplWin32 VulkanImplType;
 
-    #include <SFML/Window/Unix/WindowImplX11.hpp>
-    typedef sf::priv::WindowImplX11 WindowImplType;
+#elif defined(SFML_SYSTEM_LINUX) || defined(SFML_SYSTEM_FREEBSD) || defined(SFML_SYSTEM_OPENBSD) || defined(SFML_SYSTEM_NETBSD)
+
+    #if defined(SFML_USE_DRM)
+
+        #include <SFML/Window/DRM/WindowImplDRM.hpp>
+        typedef sf::priv::WindowImplDRM WindowImplType;
+
+        #define SFML_VULKAN_IMPLEMENTATION_NOT_AVAILABLE
+
+    #else
+
+        #include <SFML/Window/Unix/WindowImplX11.hpp>
+        typedef sf::priv::WindowImplX11 WindowImplType;
+
+        #include <SFML/Window/Unix/VulkanImplX11.hpp>
+        typedef sf::priv::VulkanImplX11 VulkanImplType;
+
+    #endif
 
 #elif defined(SFML_SYSTEM_MACOS)
 
     #include <SFML/Window/OSX/WindowImplCocoa.hpp>
     typedef sf::priv::WindowImplCocoa WindowImplType;
 
+    #define SFML_VULKAN_IMPLEMENTATION_NOT_AVAILABLE
+
 #elif defined(SFML_SYSTEM_IOS)
 
     #include <SFML/Window/iOS/WindowImplUIKit.hpp>
     typedef sf::priv::WindowImplUIKit WindowImplType;
 
+    #define SFML_VULKAN_IMPLEMENTATION_NOT_AVAILABLE
+
 #elif defined(SFML_SYSTEM_ANDROID)
 
     #include <SFML/Window/Android/WindowImplAndroid.hpp>
     typedef sf::priv::WindowImplAndroid WindowImplType;
 
+    #define SFML_VULKAN_IMPLEMENTATION_NOT_AVAILABLE
+
 #endif
 
 
@@ -65,6 +88,7 @@ namespace sf
 {
 namespace priv
 {
+
 ////////////////////////////////////////////////////////////
 WindowImpl* WindowImpl::create(VideoMode mode, const String& title, Uint32 style, const ContextSettings& settings)
 {
@@ -169,7 +193,6 @@ void WindowImpl::processJoystickEvents()
         // Copy the previous state of the joystick and get the new one
         JoystickState previousState = m_joystickStates[i];
         m_joystickStates[i] = JoystickManager::getInstance().getState(i);
-        JoystickCaps caps = JoystickManager::getInstance().getCapabilities(i);
 
         // Connection state
         bool connected = m_joystickStates[i].connected;
@@ -187,6 +210,8 @@ void WindowImpl::processJoystickEvents()
 
         if (connected)
         {
+            JoystickCaps caps = JoystickManager::getInstance().getCapabilities(i);
+
             // Axes
             for (unsigned int j = 0; j < Joystick::AxisCount; ++j)
             {
@@ -195,7 +220,7 @@ void WindowImpl::processJoystickEvents()
                     Joystick::Axis axis = static_cast<Joystick::Axis>(j);
                     float prevPos = m_previousAxes[i][axis];
                     float currPos = m_joystickStates[i].axes[axis];
-                    if (fabs(currPos - prevPos) >= m_joystickThreshold)
+                    if (std::abs(currPos - prevPos) >= m_joystickThreshold)
                     {
                         Event event;
                         event.type = Event::JoystickMoved;
@@ -261,6 +286,24 @@ void WindowImpl::processSensorEvents()
     }
 }
 
+
+////////////////////////////////////////////////////////////
+bool WindowImpl::createVulkanSurface(const VkInstance& instance, VkSurfaceKHR& surface, const VkAllocationCallbacks* allocator)
+{
+#if defined(SFML_VULKAN_IMPLEMENTATION_NOT_AVAILABLE)
+
+    (void) instance;
+    (void) surface;
+    (void) allocator;
+    return false;
+
+#else
+
+    return VulkanImplType::createVulkanSurface(instance, getSystemHandle(), surface, allocator);
+
+#endif
+}
+
 } // namespace priv
 
 } // namespace sf
